/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 *                                                                         *
 * .                                                             *
 *
 *
 * 1 - 23 - RB5
 * 2 - 28 - Q2
 * 3 - 24 - RB6
 * 4 - 29 - Q3
 * 5 - 25 - RB7
 * 6 - 30 - Q4
 * 7 - 27 - Q1
 *                                                                         *
 ***************************************************************************/

#include <pic18fregs.h>
#include "main.h"
#include "kbd.h"
#include "cd4094.h"

extern kbd_user_event( word data );

byte kbd_cnt, kbd_row;
word kbd_state, kbd_old, kbd_data;

#define KEY_F21			0x70
#define KEY_F22			0x71
#define KEY_F23			0x72
#define KEY_F24			0x73

#define KEY_VOLUMEUP		0x80
#define KEY_VOLUMEDOWN		0x81
#define KEY_PLAYPAUSE		0xE8
#define KEY_STOPCD		0xE9
#define KEY_PREVIOUSSONG	0xEA
#define KEY_NEXTSONG		0xEB

#define K_1		KEY_F21
#define K_2		KEY_F22
#define K_3		KEY_F23
#define K_4		KEY_F24

#define K_REV		KEY_PLAYPAUSE
#define K_PREV		KEY_PREVIOUSSONG
#define K_NEXT		KEY_NEXTSONG

#define K_U		0x00
#define K_M		0x00
#define K_VF		0x00
#define K_DX		0x00

#define K_UP		KEY_VOLUMEUP
#define K_DN		KEY_VOLUMEDOWN

#define K_DOLBY		0x00
#define K_LD		0x00

const byte kbd_keycode[16] =
{
    /* 		RB5		RB6		RB7 */
    /*    */	K_PREV,		K_NEXT,		K_REV,
    /* Q4 */	K_U,		K_UP,		K_4,
    /* Q3 */	K_DX,		K_DOLBY,	K_3,
    /* Q2 */	K_M,		K_LD,		K_2,
    /* Q1 */	K_VF,		K_DN,		K_1,
    0
};

void kbd_set_row( byte row )
{
    if( row < 4 )
    {
        PORTCbits.RC1 = 1;
	cd4094_set( ( cd4094_state | KBD_RMSK ) & ~( 1 << row ) );
    }
    else
    {
	cd4094_set( ( cd4094_state | KBD_RMSK ) );
        PORTCbits.RC1 = 0;
    }
}

void kbd_init( void )
{

    kbd_data  = 0xFFFF;
    kbd_state = 0xFFFF;
    kbd_old   = 0xFFFF;
    kbd_cnt   = KBD_DEBOUNCE;
    kbd_row   = 0;

    TRISCbits.TRISC1 = 0;
    kbd_set_row( kbd_row );
}

/* Every 1 mS */
void kbd_process( void )
{
    word data;

    data = kbd_data;
    data <<= KBD_COLS;
    data &= ~KBD_CMSK;
    data |= KBD_DATA;
    kbd_data = data;

    /* next row */
    kbd_row++;
    if( kbd_row >= KBD_ROWS ) kbd_row = 0;

    kbd_set_row( kbd_row );

    if( kbd_row ) return;

    /* we got here if all rows scanned ( every 4 mS ) */

    /* debounce */
    if( ( data == kbd_state ) || ( --kbd_cnt == 0 ) )
    {
	kbd_cnt   = KBD_DEBOUNCE;
	kbd_old   = kbd_state;
	kbd_state = data;
    }

    if( kbd_state != kbd_old )
    { /* something has changed in matrix */
	kbd_user_event( data );
	kbd_old = data;
    }

    kbd_data = 0xFFFF;
}
