/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 *                                                                         *
 * Audi Gamma III LCD serial interface.                                    *
 *                                                                         *
 **************************************************************************/

#include <pic18fregs.h>
#include "main.h"
#include "lcd.h"
#include "delay.h"

#define RX_CB_FREQ_MIN		26960 /* C01R 26.960 MHz */
#define RX_CB_FREQ_MAX		27405 /* C40E 27.405 MHz */

__code byte lcd_CB_chan[] =
{
/* 26,965 */    1,
/* 26,975 */    2,
/* 26,985 */    3,
/* 26,995 */    0,
/* 27,005 */    4,
/* 27,015 */    5,
/* 27,025 */    6,
/* 27,035 */    7,
/* 27,045 */    0,
/* 27,055 */    8,
/* 27,065 */    9,
/* 27,075 */    10,
/* 27,085 */    11,
/* 27,095 */    0,
/* 27,105 */    12,
/* 27,115 */    13,
/* 27,125 */    14,
/* 27,135 */    15,
/* 27,145 */    0,
/* 27,155 */    16,
/* 27,165 */    17,
/* 27,175 */    18,
/* 27,185 */    19,
/* 27,195 */    0,
/* 27,205 */    20,
/* 27,215 */    21,
/* 27,225 */    22,
/* 27,235 */    23,
/* 27,245 */    24,
/* 27,255 */    25,
/* 27,265 */    26,
/* 27,275 */    27,
/* 27,285 */    28,
/* 27,295 */    29,
/* 27,305 */    30,
/* 27,315 */    31,
/* 27,325 */    32,
/* 27,335 */    33,
/* 27,345 */    34,
/* 27,355 */    35,
/* 27,365 */    36,
/* 27,375 */    37,
/* 27,385 */    38,
/* 27,395 */    39,
/* 27,405 */    40,
};

#define SEG_A	0x40
#define SEG_B	0x20
#define SEG_C	0x10
#define SEG_D	0x08
#define SEG_E	0x04
#define SEG_F	0x01
#define SEG_G	0x02
#define SEG_DP	0x80

__code byte seg7[] = {
/* 0 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,
/* 1 */		SEG_B | SEG_C,
/* 2 */		SEG_A | SEG_B | SEG_G | SEG_E | SEG_D,
/* 3 */		SEG_A | SEG_B | SEG_G | SEG_C | SEG_D,
/* 4 */		SEG_B | SEG_C | SEG_F | SEG_G,
/* 5 */		SEG_A | SEG_F | SEG_G | SEG_C | SEG_D,
/* 6 */		SEG_A | SEG_F | SEG_G | SEG_C | SEG_D | SEG_E,
/* 7 */		SEG_A | SEG_B | SEG_C,
/* 8 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,
/* 9 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G
};

__data byte lcd_data[8];
__data byte lcd_rssi;
__data byte lcd_1;
__data byte lcd_band;
__data word lcd_freq;

byte lcd_seg7( byte d ) __wparam
{
    byte s = 0;
    if( d < sizeof( seg7 ) ) s = seg7[d];
    return s;
}

void lcd_write_byte( byte d ) __wparam
{
    byte i;

    for( i = 0; i < 8; i++ )
    {
	if( d & 1 ) LCD_DATA = 1;
	else        LCD_DATA = 0;
	LCD_CLK = 1;
	DELAY_uS(5);
	LCD_CLK = 0;
	DELAY_uS(5);
	d >>= 1;
    }
    LCD_DATA = 0;
}

void lcd_write_data( void )
{
    byte i;

    LCD_ENA = 1;
    DELAY_uS(5);
    for( i = 0; i < 8; i++ )
    {
	lcd_write_byte( lcd_data[i] );
    }
    LCD_ENA = 0;
}

void lcd_send_data( __data byte * d )
{
    byte i;

    for( i = 1; i < 8; i++ )
    {
	lcd_data[i] = *d++;
    }
    lcd_write_data();
}

void lcd_stereo( byte d ) __wparam
{
    lcd_data[1] &= ~0x02;
    if( d ) lcd_data[1] |= 0x02;
    lcd_write_data();
    lcd_1 = lcd_data[1];
}

void lcd_radio_rssi( byte rssi ) __wparam
{
    if( rssi > 9 ) rssi = 9;
    if( lcd_rssi != rssi )
    {
	lcd_data[2] = lcd_seg7( rssi );
	lcd_write_data();
	lcd_rssi = rssi;
    }
}

void lcd_radio_dx( byte d ) __wparam
{
    lcd_data[1] &= ~0x01;
    if( d ) lcd_data[1] |= 0x01;
    lcd_write_data();
    lcd_1 = lcd_data[1];
}

void lcd_set_mem( byte addr ) __wparam
{
    lcd_data[3] = 0;
    lcd_data[4] = SEG_G;
    lcd_data[5] = lcd_seg7( addr );
    lcd_data[6] = SEG_G;
    lcd_write_data();
}

void lcd_reset( void )
{
    lcd_radio( lcd_band, lcd_freq );
}

/* set LCD data to display integer v using LCD digits 3,4,5,6
 * digit 3 can only show 1 or blank, so, only 0 - 1999 range can be displayed */
void lcd_itod3456( word v )
{
    byte i;

    for( i = 6; i >= 3; i-- )
    {
	if( v )	lcd_data[i] = lcd_seg7( v % 10 );
	else	lcd_data[i] = 0;
	v /= 10;
    }
}

void lcd_radio( byte band, word freq )
{
    byte i;

    lcd_band = band;
    lcd_freq = freq;

    switch( band )
    {
	case 0: /* M */
	    lcd_data[1] = 0x2C | ( lcd_1 & 0x01 );
	    lcd_data[2] = lcd_seg7( lcd_rssi );

	    i = 0;

	    if( ( freq >= RX_CB_FREQ_MIN ) && ( freq <= RX_CB_FREQ_MAX ) && ( ( freq % 5 ) == 0 ) )
	    { /* may be CB channel */
		i = lcd_CB_chan[ ( freq - RX_CB_FREQ_MIN ) / 10 ];
		if( i )
		{
		    lcd_data[3] = 0;
		    lcd_data[4] = SEG_A | SEG_D | SEG_E | SEG_F; /* C */
		    lcd_data[6] = lcd_seg7( i % 10 ); i /= 10;
		    lcd_data[5] = lcd_seg7( i % 10 );
		    if( freq % 10 )	lcd_data[7] = 0x07; /* E channel */
		    else		lcd_data[7] = 0x01; /* R */
		    i = 1; /* display ready */
		}
	    }

	    if( !i )
	    { /* just some frequency */
		freq -= 26000;
		lcd_data[7] = 0;
		lcd_itod3456( freq );
	    }

	    break;

	case 1: /* U */
	    lcd_data[1] = 0x38 | ( lcd_1 & 0x03 );
	    lcd_data[2] = lcd_seg7( lcd_rssi );
	    lcd_itod3456( freq / 10 );
	    lcd_data[5] |= SEG_DP;
	    lcd_data[7] = 0;
	    break;

	case 2:  /* PCM */
	    lcd_data[1] = 0x02; /* stereo */
	    lcd_data[2] = 0; /* rssi */
	    lcd_data[3] = 0;
	    for( i = 6; i >= 4; i-- )
	    {
		lcd_data[i] = lcd_seg7( freq % 10 );
		freq /= 10;
	    }
	    lcd_data[7] = 0x10; /* VF */
	    break;
    }

    lcd_write_data();
}

void lcd_init( void )
{
    byte i;

    TRISBbits.TRISB0 = 0;
    TRISBbits.TRISB1 = 0;
    TRISBbits.TRISB2 = 0;

    LCD_ENA = 0;

    i = 8; while(i) lcd_data[--i] = 0;

    lcd_write_data();

    lcd_1 = 0;
    lcd_band = 1;
    lcd_freq = 9520;
}
