/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 **************************************************************************/
#define extern
#include <pic18fregs.h>
#undef extern
#include "main.h"
#include "usb.h"
#include "config.h"

#include "button.h"
#include "eeprom.h"
#include "lcd.h"
#include "delay.h"
#include "mux.h"
#include "cd4094.h"
#include "rx.h"
#include "saa1057.h"
#include "kbd.h"

byte chr_cnt;
byte chr_done;
byte *chr_ptr;

word delay_ms;
word delay_sec;

byte mode;

#define MODE_MUTE	0
#define MODE_PCM	1
#define MODE_RADIO	2

byte display_timeout;

byte key_last;
byte key_timeout;
#define KBD_TIMEOUT	3 /* 3 sec keyboard timeout */

word key_state, key_old;

#define LOW(x) (((word)(x)) & 0xFF)
#define HIGH(x) ((((word)(x)) >> 8) & 0xFF)
#define SET_TMR0_uS(x) { TMR0H = HIGH(-(x) * 3); TMR0L = LOW(-(x) * 3); }

#pragma stack 0x200 255

#pragma code _reset 0x800
void _reset (void) __naked
{
    __asm
    GOTO _main
    __endasm;
}

#pragma code _high_ISR 0x808
void _high_ISR (void) __naked
{
    __asm
    GOTO _high_interrupt
    __endasm;
}

#pragma code _low_ISR 0x818
void _low_ISR (void) __naked
{
    __asm
    GOTO _low_interrupt
    __endasm;
}


void high_interrupt(void) __interrupt
{
}

/*  .     */
void low_interrupt(void) __interrupt
{
    if( PIR2bits.USBIF ) USBIntService();
}

void mode_select( __data byte * m )
{
    byte fm;

    mode = *m++;
    switch( mode )
    {
	case MODE_MUTE:
	    mux_select( MUX_MUTE );
	    break;
	case MODE_PCM:
	    mux_select( MUX_PCM );
	    break;
	case MODE_RADIO:
	    mux_select( MUX_RADIO );
	    fm = *m++;
	    rx_setup( fm, *(word *)m );
	    break;
    };
}

void btn_user_event( byte state )
{ /*    */
    if( mode == MODE_RADIO )
    { /* start scanning */
	if( state ) rx_scan_start();
	else        rx_scan_stop();
    }
}

void kbd_user_event( word data )
{
    word diff;
    byte key, off;

    key_state = data;

    if( mode != MODE_RADIO )
    {
	diff = data ^ key_old;
	if( diff & data & ( 1 << ( KBD_K_VF - 1 ) ) )
	{
	    mode = MODE_RADIO;
	    mux_select( MUX_RADIO );
	    rx_tune(0);
	}
	return;
    }

    diff = data ^ key_old;
    for( key = 1; diff; diff >>= 1, data >>= 1, key++ )
    {
	if( !( diff & 1 ) ) continue;

	if( data & 1 )
	{ /* key released */
	    if( key_timeout )
	    { /* fast key press */
		off = 0xFF;
		switch( key )
		{
		    case KBD_K_VF:
			mode = MODE_PCM;
			mux_select( MUX_PCM );
			lcd_radio( 2, 0 );
			break;
		    case KBD_K_1: off = 0; break;
		    case KBD_K_2: off = 1; break;
		    case KBD_K_3: off = 2; break;
		    case KBD_K_4: off = 3; break;
			break;
		    default:
		        break;
		}
		if( off < 4 )
		{
		    rx_restore( off );
		}
	    }
	    key_last = 0;
	}
	else
	{ /* key pressed */
	    switch( key )
	    {
		case KBD_K_M:
		    rx_set_fm(0);
		    break;
		case KBD_K_U:
		    rx_set_fm(1);
		    break;
		case KBD_K_DX:
		    rx_dx_toggle();
		    break;
		case KBD_K_UP:
		    rx_tune( +1 );
		    key_last = key;
		    break;
		case KBD_K_DN:
		    rx_tune( -1 );
		    break;
		default:
		    break;
	    }
	    key_timeout = KBD_TIMEOUT;
	    key_last = key;
	}
    }

    key_old = key_state;
}

void user_out(void)
{
    if( USER_BD_OUT.Stat & _USIE ) return;

    /*     */

    switch( USER_BD_OUT.ADR[0] )
    {
	case 0: lcd_send_data( &USER_BD_OUT.ADR[1] ); break;
	case 1: mode_select( &USER_BD_OUT.ADR[1] );   break;
    }

    /*    */
    USER_BD_OUT.Cnt   = USER_EP_SIZE;
    USER_BD_OUT.Stat &= _DTSMASK;
    USER_BD_OUT.Stat ^= _DTSMASK;
    USER_BD_OUT.Stat |= _USIE | _DTSEN;
}

void user_in(void)
{
    if( USER_BD_IN.Stat & _USIE ) return;

    /*      */

//    if( kbd_key )
//    {
//	USER_BD_IN.ADR[0] = kbd_event;
//	USER_BD_IN.ADR[1] = kbd_key;
	/*   */
//	USER_BD_IN.Cnt   = 2;
//	USER_BD_IN.Stat &= _DTSMASK;
//	USER_BD_IN.Stat ^= _DTSMASK;
//	USER_BD_IN.Stat |= _USIE | _DTSEN;
//	kbd_key = 0;
//    }
}

void user_in_hid( byte type, byte id )
{
    byte i, k, c;
    word d;

    type; id;

    if( HID_BD_IN.Stat & _USIE ) return;

    if( key_state != key_old )
    { /* keyboard state changed, send report */

	i = 8; while(i) HID_BD_IN.ADR[--i] = 0;

	i = 2;

	for( d = key_state, k = 0; ( k < 16 ) && ( i < 8 ); d >>= 1, k++ )
	{
	    if( !( d & 1 ) )
	    { /* key pressed */
		c = kbd_keycode[k];
		if( c ) HID_BD_IN.ADR[i++] = c;
	    }
	}
	/* Send keyboard data */
	HID_BD_IN.Cnt   = 8;
	HID_BD_IN.Stat &= _DTSMASK;
	HID_BD_IN.Stat ^= _DTSMASK;
	HID_BD_IN.Stat |= _USIE | _DTSEN;

	key_old = key_state;
    }
}

void user_out_hid( byte type, byte id ) { type; id; }

void UserService( void )
{
    byte off;

    if(INTCONbits.TMR0IF)
    { /*   (1 mS) */

	/*     1000  */
	SET_TMR0_uS(1000);
	INTCONbits.TMR0IF = 0;

	if(--delay_ms == 0)
	{
	    delay_ms = 1000;

	    if(--delay_sec == 0)
	    {
		delay_sec = 10;
	    }

	    if( display_timeout )
	    {
		display_timeout--;
		if( ! display_timeout )
		{
		    lcd_reset();
		}
	    }

	    if( key_timeout )
	    {
		key_timeout--;
		if( ! key_timeout )
		{
		    off = 0xFF;
		    switch( key_last )
		    {
			case KBD_K_1: off = 0; break;
			case KBD_K_2: off = 1; break;
			case KBD_K_3: off = 2; break;
			case KBD_K_4: off = 3; break;
			default: break;
		    }
		    if( off < 4 )
		    {
			rx_save( off );
			lcd_set_mem( off + 1 );
			display_timeout = 1;
		    }
		}
	    }
	}

	btn_process();
	kbd_process();
	if( mode == MODE_RADIO ) rx_process();
    }

    if( ( usb_device_state == CONFIGURED_STATE ) && !( UCONbits.SUSPND ) )
    {
	user_in_hid( 0, 0 );
	user_out();
	user_in();
    }
}

void mode_init( void )
{
    mode = MODE_RADIO;
    mux_select( MUX_RADIO );
    rx_dx_set( 1 );
    rx_restore( 0 );
}

void main(void)
{

    __asm
    LFSR 1, _stack_end
    LFSR 2, _stack_end
    __endasm;

    /*   */

    ADCON1 = 0x0F;
    CMCON = 0x07;

    PORTA = 0xFF;
    TRISA = 0xFF;

    PORTB = 0xFF;
    TRISB = 0xFF;

    PORTC = 0xFF;
    TRISC = 0xFF;

    INTCON2bits.RBPU = 0;

    /*   (16 , CLK 3  (48  / 4 / 4) 0.333 ) */
    T0CON = 0x81;
    INTCONbits.TMR0IF = 0;
    SET_TMR0_uS(1000);

    delay_ms = 1000;
    delay_sec = 10;

    chr_cnt = 0;

    mode = MODE_MUTE;

    display_timeout = 0;

    key_last    = 0;
    key_timeout = 0;
    key_state   = 0xFFFF;
    key_old     = 0xFFFF;

    delay_init();
    btn_init();
    lcd_init();
    mux_init();
    cd4094_init();
    saa1057_init();
    rx_init();
    kbd_init();

    mode_init();

    /*  USB */
    USBInit();

    RCONbits.IPEN   = 1;	/* Enable interrupt priority */
    INTCONbits.GIEL = 1;	/* Enable low priority interrupts */
    INTCONbits.GIEH = 1;	/* Enable high priority interrupts */

    /* Read EEPROM */

    while(1)
    {
	UserService();
    }
}
