/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 *                                                                         *
 * Receiver high-level control                                             *
 *                                                                         *
 **************************************************************************/

#include <pic18fregs.h>
#include "main.h"
#include "cd4094.h"
#include "lcd.h"
#include "saa1057.h"
#include "rx.h"
#include "eeprom.h"

extern rx_user_event(void);

byte rx_dx;
byte rx_fm;
word rx_freq, rx_am_freq, rx_fm_freq;

byte rx_stereo_state, rx_stereo_old, rx_stereo_cnt;

byte rx_scan_cnt, rx_scan;

byte rx_rssi_cnt, rx_rssi;

/* every 1 mS */
void rx_process( void )
{
    word freq;

    if((RX_STEREO_PIN == rx_stereo_state) || (--rx_stereo_cnt == 0))
    {
	rx_stereo_cnt   = RX_STEREO_DEBOUNCE;
	rx_stereo_old   = rx_stereo_state;
	rx_stereo_state = RX_STEREO_PIN;
    }

    if( rx_stereo_state != rx_stereo_old )
    {
        rx_stereo_old = rx_stereo_state;
	lcd_stereo( rx_stereo_state );
    }

    if( PIR1bits.ADIF )
    {
	rx_rssi = ADRESH;
	lcd_radio_rssi( rx_rssi / 10 );
	PIR1bits.ADIF = 0;
    }

    if( --rx_rssi_cnt == 0 )
    {
	rx_rssi_cnt = RX_RSSI_PERIOD;
	PIR1bits.ADIF = 0;
	ADCON0bits.GO = 1;
    }

    if( rx_scan )
    {
	if( --rx_scan_cnt == 0 )
	{
	    freq = rx_freq;
	    rx_scan_cnt = RX_SCAN_PERIOD;
	    if( rx_fm )
	    {
		freq += 10;
		if( freq > RX_FM_FREQ_MAX ) freq = RX_FM_FREQ_MIN;
	    }
	    else
	    {
		freq += 5;
		if( freq > RX_AM_FREQ_MAX ) freq = RX_AM_FREQ_MIN;
	    }
	    saa1057_freq( freq );
	    lcd_radio( rx_fm, freq );
	    rx_freq = freq;
	}
    }
}

void rx_dx_set( byte dx ) __wparam
{
    rx_dx = dx;
    if( dx ) RX_DX_PIN = 0;
    else     RX_DX_PIN = 1;
    lcd_radio_dx( dx );
}

void rx_dx_toggle( void )
{
    rx_dx_set( !rx_dx );
}

void rx_setup( byte fm, word freq )
{
    if( fm )
    {
	if( freq < RX_FM_FREQ_MIN ) freq = RX_FM_FREQ_MIN;
	if( freq > RX_FM_FREQ_MAX ) freq = RX_FM_FREQ_MAX;
    	cd4094_bit_set( RX_FM_PIN );
    }
    else
    {
	if( freq < RX_AM_FREQ_MIN ) freq = RX_AM_FREQ_MIN;
	if( freq > RX_AM_FREQ_MAX ) freq = RX_AM_FREQ_MAX;
    	cd4094_bit_clr( RX_FM_PIN );
    }

    saa1057_set( fm, 0, freq );
    lcd_radio( fm, freq );

    rx_fm = fm;
    rx_freq = freq;
}

void rx_set_fm( byte fm ) __wparam
{
    word freq;

    if( fm == rx_fm ) return;

    freq = rx_freq;

    if( fm )
    {
    	rx_am_freq = freq;
    	freq = rx_fm_freq;
    }
    else
    {
    	rx_fm_freq = freq;
    	freq = rx_am_freq;
    }

    rx_setup( fm, freq );
}


void rx_save( byte off ) __wparam
{
    if( !rx_fm ) off += 4;
    eeprom_write_data( off * 2, (__data byte *)&rx_freq, 2 );
}

void rx_restore( byte off ) __wparam
{
    if( !rx_fm ) off += 4;
    eeprom_read_data( off * 2 , (__data byte *)&rx_freq, 2 );
    rx_setup( rx_fm, rx_freq );
}

void rx_tune( char step ) __wparam
{
    if( rx_fm ) step *= 10;
    else	step *= 5;
    rx_freq += step;
    rx_setup( rx_fm, rx_freq );
}

void rx_scan_start( void )
{
    rx_scan_cnt = RX_SCAN_PERIOD;
    rx_scan = 1;
    saa1057_set( rx_fm, rx_scan, rx_freq );
}

void rx_scan_stop( void )
{
    rx_scan = 0;
}

void rx_init( void )
{
    RX_STEREO_TRIS = 1;
    RX_DX_TRIS     = 0;

    ADCON1          = 0x0C; /* AN0 - RSSI, AN1 - AFC Vref, AN2 - AFC Vout */
    ADCON2          = 0x2E; /* 64Tosc = 1.333uS, 12Tad = 16uS */
    ADCON0          = 0x01;
    PIR1bits.ADIF   = 0;

    rx_dx           = 1;
    RX_DX_PIN       = 0;

    rx_fm           = 1;
    cd4094_bit_set( RX_FM_PIN );

    rx_am_freq      = 1000; /* 1MHz */
    rx_fm_freq      = 9520; /* 95.2 MHz */
    rx_freq         = rx_fm_freq;

    rx_stereo_state = 0;
    rx_stereo_old   = 0;
    rx_stereo_cnt   = RX_STEREO_DEBOUNCE;

    rx_rssi_cnt     = RX_RSSI_PERIOD;
    rx_rssi         = 0;

    rx_scan_cnt     = RX_SCAN_PERIOD;
    rx_scan         = 0;
}
