/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 **************************************************************************/
#include <pic18fregs.h>
#include "main.h"
#include "usb.h"

__data __at (0x400) BDT ep0Bo;      //Endpoint #0 BD Out
__data __at (0x404) BDT ep0Bi;      //Endpoint #0 BD In
__data __at (0x408) BDT ep1Bo;	//Endpoint #1 BD Out
__data __at (0x40C) BDT ep1Bi;	//Endpoint #1 BD In
__data __at (0x410) BDT ep2Bo;	//Endpoint #2 BD Out
__data __at (0x414) BDT ep2Bi;	//Endpoint #2 BD In

#pragma udata bank4 SetupPkt
__data CTRL_TRF_SETUP   SetupPkt;
#pragma udata bank4 ctrl_trf_data
__data byte ctrl_trf_data[EP0_BUFF_SIZE];
#pragma udata bank4 ctrl_trf_state
__data byte ctrl_trf_state;            // Control Transfer State
#pragma udata bank4 ctrl_trf_count
__data word ctrl_trf_count;            // Data counter
#pragma udata bank4 ctrl_trf_mem
__data byte ctrl_trf_mem;

#pragma udata bank4 pSrc
__data POINTER pSrc;                   // Data source pointer
#pragma udata bank4 pDst
__data POINTER pDst;                   // Data destination pointer

#pragma udata bank4 usb_device_state
__data byte usb_device_state;          // Device States: DETACHED, ATTACHED, ...
#pragma udata bank4 usb_active_cfg
__data byte usb_active_cfg;            // Value of current configuration
#pragma udata bank4 usb_new_addr
__data byte usb_new_addr;              // Address sent by host
#pragma udata bank4 usb_ctrl_data
__data byte usb_ctrl_data[2];

/*    endpoint' */
#pragma udata bank4 ep1_data_in
__data byte ep1_data_in[HID_EP_SIZE];
#pragma udata bank4 ep2_data_in
__data byte ep2_data_in[USER_EP_SIZE];
#pragma udata bank4 ep2_data_out
__data byte ep2_data_out[USER_EP_SIZE];

/*   */
#pragma udata bank4 idle_rate0
__data byte idle_rate0;
#pragma udata bank4 active_protocol0
__data byte active_protocol0;


extern void user_in_hid(byte type, byte id);
extern void user_out_hid(byte type, byte id);

extern void user_in(byte type, byte id);
extern void user_out(byte type, byte id);

/* descriptors */
#include "usb_descr.h"

void UserInitEP(void)
{
    /* set up all endpoints */
    HID_UEP = EP_IN | HSHK_EN;				// Enable IN data pipe

    HID_BD_IN.ADR  = (__data byte*)&ep1_data_in;	// Set buffer address
    HID_BD_IN.Stat = _UCPU | _DAT1;			// Set status

    USER_UEP = EP_OUT_IN | HSHK_EN;			// Enable 2 data pipes

    USER_BD_OUT.Cnt  = USER_EP_SIZE;			// buffer size
    USER_BD_OUT.ADR  = (__data byte*)&ep2_data_out;	// buffer address
    USER_BD_OUT.Stat = _USIE | _DAT0 | _DTSEN;		// status

    USER_BD_IN.ADR  = (__data byte*)&ep2_data_in;	// buffer address
    USER_BD_IN.Stat = _UCPU | _DAT1;			// status
}

void USBPrepareForNextSetupTrf(void)
{
    ctrl_trf_state = WAIT_SETUP;          // See usbctrltrf.h
    ep0Bo.Cnt  = EP0_BUFF_SIZE;           //  
    ep0Bo.ADR  = (__data byte*)&SetupPkt;   //  SetupPkt
    ep0Bo.Stat = _USIE | _DAT0 | _DTSEN;  // EP0 buff dsc init, see usbmmap.h
    ep0Bi.Stat = _UCPU;                   // EP0 IN buffer initialization
}

void USBCtrlTrfTxService(void)
{
    byte count;

    /* First, have to figure out how many byte of data to send. */
    if(ctrl_trf_count < EP0_BUFF_SIZE)
        count = ctrl_trf_count;
    else
        count = EP0_BUFF_SIZE;

    ep0Bi.Cnt = count;

    /* Subtract the number of bytes just about to be sent from the total. */
    ctrl_trf_count -= count;

    if(ctrl_trf_mem == _ROM)
    {
        /*   ROM */
	__asm LFSR 0, _ctrl_trf_data __endasm;
        TBLPTRL = pSrc.bLow;
        TBLPTRH = pSrc.bHigh;
        TBLPTRU = pSrc.bUpper;
        while(count)
	{
            __asm TBLRD*+ __endasm;
	    POSTINC0 = TABLAT;
	    count--;
	}
	/*   */
        pSrc.bLow   = TBLPTRL;
        pSrc.bHigh  = TBLPTRH;
        pSrc.bUpper = TBLPTRU;
    }
    else
    {
        /*   RAM */
	__data byte *src = pSrc.bRam;
	__data byte *dst = ctrl_trf_data;

        while(count) { *dst = *src; dst++; src++; count--; }

	pSrc.bRam = src;
    }
}

void USBCtrlEPServiceComplete(void)
{
    if(SetupPkt.bmRequestType & 0x80) // DEV_TO_HOST
    {
        /* Control Read */
        ctrl_trf_state = CTRL_TRF_TX;

        if(SetupPkt.wLength < ctrl_trf_count)
            ctrl_trf_count = SetupPkt.wLength;

        /*        */
        USBCtrlTrfTxService();

        /* Data stage -    */
        ep0Bi.ADR = (__data byte*)&ctrl_trf_data;

        /* Status stage -   OUT(1) */
        ep0Bo.Cnt = EP0_BUFF_SIZE;
        ep0Bo.ADR = (__data byte*)&SetupPkt;            
        ep0Bo.Stat = _USIE;           // Note: DTSEN is 0!
    }
    else // HOST_TO_DEV
    {
        /* Control Write  No-data Control */
        ctrl_trf_state = CTRL_TRF_RX;

        /* Data stage -     */
        ep0Bo.Cnt = EP0_BUFF_SIZE;
        ep0Bo.ADR = (__data byte*)&ctrl_trf_data;
        ep0Bo.Stat = _USIE | _DAT1 | _DTSEN;

        /* Status stage -   IN(1) */
        ep0Bi.Cnt = 0;
    }
}

void USBCtrlStall(void)
{
    /*  STALL     SETUP */
    ep0Bo.Cnt = EP0_BUFF_SIZE;
    ep0Bo.ADR = (__data byte*)&SetupPkt;
    ep0Bo.Stat = _USIE | _BSTALL;
    ep0Bi.Stat = _USIE | _BSTALL;
}

void USBCheckStdRequest(void)
{
    if((SetupPkt.bmRequestType & 0x60) != STANDARD) return;

    ctrl_trf_count = 0;

    switch(SetupPkt.bRequest)
    {
        case SET_ADR:
            usb_new_addr     = (byte)SetupPkt.wValue;   //   
            usb_device_state = ADR_PENDING_STATE;       // Update state only
	    break;

	case GET_STATUS:
	    usb_ctrl_data[0] = 0;                         // Initialize content
	    usb_ctrl_data[1] = 0;

	    switch(SetupPkt.bmRequestType & 0x1F)
	    {
    		case RCPT_DEV:
            	    usb_ctrl_data[0] = DEV_SELF_POWERED | (DEV_REMOTE_WAKEUP << 1);
        	    break;
    		case RCPT_INTF:
        	    break;
    		case RCPT_EP:
        	    pDst.bRam = (__data byte*)&ep0Bo;
		    pDst.bRam += (SetupPkt.wIndex & 0x0F) << 3;
		    if(SetupPkt.wIndex & 0x80) pDst.bRam += 4;
        	    if(*pDst.bRam & _BSTALL) usb_ctrl_data[0] = 1;
        	    break;
	    }

    	    pSrc.bRam      = (__data byte*)&usb_ctrl_data; // Set Source
    	    ctrl_trf_count = 2;                          // Set data count
    	    ctrl_trf_mem   = _RAM;                       // Set memory type
	    break;

        case GET_DSC:
	    switch(SetupPkt.bDscType)
	    {
                case DSC_DEV:
                    pSrc.bRom = (__code byte*)&device_dsc;
                    ctrl_trf_count = sizeof(device_dsc);
		    break;
                case DSC_CFG:
                    pSrc.bRom = (__code byte*)&cfg01;
                    ctrl_trf_count = sizeof(cfg01);
		    break;
                case DSC_STR:
		    if(SetupPkt.bDscIndex >= sizeof(string_dsc)/sizeof(string_dsc[0]))
			goto Stall;
	            pSrc.bRom      = string_dsc[SetupPkt.bDscIndex].adr;
	            ctrl_trf_count = string_dsc[SetupPkt.bDscIndex].len;
		    break;
        	case DSC_HID:
		    if(SetupPkt.wInterface >= sizeof(hid_dsc)/sizeof(hid_dsc[0]))
			goto Stall;
	            pSrc.bRom      = hid_dsc[(byte)SetupPkt.wInterface].adr;
	            ctrl_trf_count = hid_dsc[(byte)SetupPkt.wInterface].len;
            	    break;
        	case DSC_RPT:
		    if(SetupPkt.wInterface >= sizeof(hid_rpt)/sizeof(hid_rpt[0]))
			goto Stall;
	            pSrc.bRom      = hid_rpt[(byte)SetupPkt.wInterface].adr;
	            ctrl_trf_count = hid_rpt[(byte)SetupPkt.wInterface].len;
		    break;
                default: //   
		    goto Stall;
	    }
            ctrl_trf_mem = _ROM;               // Set memory type
            break;

        case SET_CFG:
            usb_active_cfg = (byte)SetupPkt.wValue;

            if( usb_active_cfg == 0 )
	    {
                usb_device_state = ADDRESS_STATE;
	    }
            else
            {
		UserInitEP();
                usb_device_state = CONFIGURED_STATE;
            }
            break;

        case GET_CFG:
            pSrc.bRam = (__data byte*)&usb_active_cfg;    // Set Source
            ctrl_trf_count = 1;                         // Set data count
            ctrl_trf_mem = _RAM;                        // Set memory type
            break;

	case SET_INTF:
	    switch(SetupPkt.wIndex)
	    {
		case 0:
		    HID_BD_IN.Stat = _UCPU | _DAT1;
		    break;
		case 1:
		    /* reinit interface? */
		    USER_BD_OUT.Stat = _USIE | _DAT0 | _DTSEN;
		    USER_BD_IN.Stat  = _UCPU | _DAT1;
		    break;
		default:
		    goto Stall;
	    }
	    break;
        default: //  
	    goto Stall;
    }
    USBCtrlEPServiceComplete();
    return;
Stall:
    USBCtrlStall();
    return;
}

void USBCheckHIDRequest(void)
{

    if((SetupPkt.bmRequestType & 0x7F) != (CLASS | RCPT_INTF)) return;

    ctrl_trf_count = 0;

    switch(SetupPkt.bRequest)
    {
        case GET_REPORT:
	    switch(SetupPkt.wInterface)
	    {
		case 0: user_in_hid(SetupPkt.bReportType, SetupPkt.bReportID); break;
	    }
            break;

        case SET_REPORT:
	    switch(SetupPkt.wInterface)
	    {
		case 0: user_out_hid(SetupPkt.bReportType, SetupPkt.bReportID); break;
	    }
            break;

        case GET_IDLE:
	    switch(SetupPkt.wInterface)
	    {
		case 0:
        	    pSrc.bRam = (__data byte*)&idle_rate0;      // Set source
        	    ctrl_trf_count = 1;        // Set data count
        	    ctrl_trf_mem = _RAM;       // Set memory type
		    break;
	    }
            break;

        case SET_IDLE:
	    switch(SetupPkt.wInterface)
	    {
        	case 0: idle_rate0 = SetupPkt.bIdleRate; break;
	    }
            break;

        case GET_PROTOCOL:
	    switch(SetupPkt.wInterface)
	    {
		case 0:
        	    pSrc.bRam = (__data byte*)&active_protocol0; // Set source
        	    ctrl_trf_count = 1;                       // Set data count
        	    ctrl_trf_mem = _RAM;                      // Set memory type
		    break;
	    }
            break;

        case SET_PROTOCOL:
	    switch(SetupPkt.wInterface)
	    {
        	case 0: active_protocol0 = SetupPkt.bActiveProtocol; break;
	    }
            break;

	default:
	    goto Stall;
    }
    USBCtrlEPServiceComplete();
    return;
Stall:
    USBCtrlStall();
    return;
}


void USBCtrlEPService(void)
{
    if(USTAT == EP00_OUT)
    {
        if((ep0Bo.Stat & TOKEN_TYPE_MASK) == SETUP_TOKEN) // EP0 SETUP
	{
            USBCheckStdRequest();
	    USBCheckHIDRequest();
            ep0Bi.Stat = _USIE | _DAT1 | _DTSEN;
            UCONbits.PKTDIS = 0;
	}
        else                                        // EP0 OUT
	{
	    if(ctrl_trf_state == CTRL_TRF_RX)
	    {
		ep0Bo.Stat &= _DTSMASK;
		ep0Bo.Stat ^= _DTSMASK;
		ep0Bo.Stat |= _USIE | _DTSEN;
	    }
	    else
	    {
	        USBPrepareForNextSetupTrf();
	    }
        }
    }
    else if(USTAT == EP00_IN)                       // EP0 IN
    {
	if(usb_device_state == ADR_PENDING_STATE)
        {
            UADDR = usb_new_addr;
            if(UADDR)
                usb_device_state = ADDRESS_STATE;
	    else
                usb_device_state = DEFAULT_STATE;
        }

        if(ctrl_trf_state == CTRL_TRF_TX)
        {
            USBCtrlTrfTxService();

	    ep0Bi.Stat &= _DTSMASK;
	    ep0Bi.Stat ^= _DTSMASK;
	    ep0Bi.Stat |= _USIE | _DTSEN;
        }
        else
	{
           USBPrepareForNextSetupTrf();
	}
    }
}

void USBIntService(void)
{
    byte flags = UIR & UIE;

    PIR2bits.USBIF = 0;

    if(flags & _SOFIF)
    {
	UIRbits.SOFIF = 0;
    }

    if(flags & _ACTIVIF)
    {
        UCONbits.SUSPND = 0;
        UIEbits.ACTVIE = 0;
	while(UIRbits.ACTVIF) { UIRbits.ACTVIF = 0; }
    }

    if(UCONbits.SUSPND) return;

    if(flags & _URSTIF)
    {
        while(UIRbits.TRNIF)            // Flush any pending transactions
            UIRbits.TRNIF = 0;
        UIR   = 0;                      // Clears all USB interrupts
        UIE   = 0x6B;                   // Enable all interrupts except ACTVIE & IDLE
        UADDR = 0;                      // Reset to default address
        UEP0  = EP_CTRL | HSHK_EN;      // Init EP0 as a Ctrl EP
        USBPrepareForNextSetupTrf();    //  BDT  ɣ SETUP
        UCONbits.PKTDIS = 0;            // Make sure packet processing is enabled

        usb_active_cfg   = 0;           // Clear active configuration
        usb_device_state = DEFAULT_STATE;
    }

    if(flags & _IDLEIF)
    {
        UIEbits.ACTVIE = 1;                    // Enable bus activity interrupt
        UIRbits.IDLEIF  = 0;
        UCONbits.SUSPND = 1;                    // Put USB module in power conserve
    }

    if(flags & _STALLIF)
    {
        if(UEP0bits.EPSTALL)
        {
            USBPrepareForNextSetupTrf();        // Firmware Work-Around
            UEP0bits.EPSTALL = 0;
        }
        UIRbits.STALLIF = 0;
    }

    if(usb_device_state < DEFAULT_STATE) return;

    for(flags = 0; flags < 4; flags++)
    {
        if(UIRbits.TRNIF && UIEbits.TRNIE)
	{
	    USBCtrlEPService();
	    UIRbits.TRNIF = 0;
	}
	else break;
    }
}

void USBInit(void)
{
    ep0Bo.Stat = _UCPU;
    ep0Bi.Stat = _UCPU;
    ep1Bo.Stat = _UCPU;
    ep1Bi.Stat = _UCPU;
    ep2Bo.Stat = _UCPU;
    ep2Bi.Stat = _UCPU;

    UCFG = _PUEN | _TRINT | _FS | _PPBM0;
    UCON = 0x08;                          // Enable USBEN only
    while(UCONbits.SE0);                  // Blocking loop
    UIR = 0;                              // Clear all USB interrupts
    UIE = 0x11;                           // Unmask RESET & IDLE interrupts only

    usb_device_state = POWERED_STATE;

    IPR2bits.USBIP = 0;		// Set USB interrupt priority low
    PIR2bits.USBIF = 0;		// Clear USB interrupt flag
    PIE2bits.USBIE = 1;		// Enable USB interrupts
}
