/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 **************************************************************************/

#define DEV_SELF_POWERED 	0
#define DEV_REMOTE_WAKEUP 	0

/* Device Descriptor */

typedef struct _USB_DEV_DSC
{
    byte bLength;
    byte bDscType;
    word bcdUSB;
    byte bDevCls;
    byte bDevSubCls;
    byte bDevProtocol;
    byte bMaxPktSize0;
    word idVendor;
    word idProduct;
    word bcdDevice;
    byte iMFR;
    byte iProduct;
    byte iSerialNum;
    byte bNumCfg;
} USB_DEV_DSC;

const USB_DEV_DSC device_dsc = 
{
    sizeof(USB_DEV_DSC),    // Size of this descriptor in bytes
    DSC_DEV,                // DEVICE descriptor type
    0x0200,                 // USB Spec Release Number in BCD format
    0x00,                   // Class Code
    0x00,                   // Subclass code
    0x00,                   // Protocol code
    EP0_BUFF_SIZE,          // Max packet size for EP0
    0xdead,                 // Vendor ID: Illegal Systems Corp.
    0x0034,                 // Product ID: Template
    0x0101,                 // Device release number in BCD format
    0x01,                   // Manufacturer string index
    0x02,                   // Product string index
    0x03,                   // Device serial number string index
    0x01                    // Number of possible configurations
};

/* Configuration Descriptor */

typedef struct _USB_CFG_DSC
{
    byte bLength;
    byte bDscType;
    word wTotalLength;
    byte bNumIntf;
    byte bCfgValue;
    byte iCfg;
    byte bmAttributes;
    byte bMaxPower;
} USB_CFG_DSC;

typedef struct _USB_INTF_DSC
{
    byte bLength;
    byte bDscType;
    byte bIntfNum;
    byte bAltSetting;
    byte bNumEPs;
    byte bIntfCls;
    byte bIntfSubCls;
    byte bIntfProtocol;
    byte iIntf;
} USB_INTF_DSC;

typedef struct _USB_HID_DSC_HEADER
{
    byte bDscType;
    word wDscLength;
} USB_HID_DSC_HEADER;

typedef struct _USB_HID_DSC
{
    byte bLength;
    byte bDscType;
    word bcdHID;
    byte bCountryCode;
    byte bNumDsc;
    USB_HID_DSC_HEADER hid_dsc_header[1];
} USB_HID_DSC;

typedef struct _USB_EP_DSC
{
    byte bLength;
    byte bDscType;
    byte bEPAdr;
    byte bmAttributes;
    word wMaxPktSize;
    byte bInterval;
} USB_EP_DSC;

typedef struct _USB_CONFIGURATION
{
    USB_CFG_DSC             cd01;

    USB_INTF_DSC            i00a00;
    USB_HID_DSC             hid_i00a00;
    USB_EP_DSC              ep01i_i00a00;

    USB_INTF_DSC            i01a00;
    USB_EP_DSC              ep02i_i01a00;
    USB_EP_DSC              ep02o_i01a00;

} USB_CONFIGURATION;

const USB_CONFIGURATION cfg01 =
{
    {
        sizeof(USB_CFG_DSC),      // Size of this descriptor in bytes
        DSC_CFG,                  // CONFIGURATION descriptor type
        sizeof(cfg01),            // Total length of data for this cfg
        2,                        // Number of interfaces in this cfg
        1,                        // Index value of this configuration
        0,                        // Configuration string index
        _DEFAULT,                 // Attributes, see usbdefs_std_dsc.h
        250                       // Max power consumption (2X mA)
    },

    /* Interface Descriptor */
    {
        sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
        DSC_INTF,               // INTERFACE descriptor type
        0,                      // Interface Number
        0,                      // Alternate Setting Number
        1,                      // Number of endpoints in this intf
        HID_INTF,               // Class code
        BOOT_INTF_SUBCLASS,     // Subclass code
        HID_PROTOCOL_KEYBOARD,  // Protocol code
        4                       // Interface string index
    },
    /* HID Class-Specific Descriptor */
    {
	sizeof(USB_HID_DSC),    // Size of this descriptor in bytes
	DSC_HID,                // HID descriptor type
	0x0101,                 // HID Spec Release Number in BCD format
	0x00,                   // Country Code (0x00 for Not supported)
	1,         		// Number of class descriptors
	{
	    {
		DSC_RPT,            // Report descriptor type
	        sizeof(hid_rpt00)   // Size of the report descriptor
	    }
	}
    },
    /* Endpoint Descriptor */
    { sizeof(USB_EP_DSC), DSC_EP, _EP01_IN, _INT, HID_EP_SIZE, 0x0A },

    /* Interface Descriptor */
    {
        sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
        DSC_INTF,               // INTERFACE descriptor type
        1,                      // Interface Number
        0,                      // Alternate Setting Number
        2,                      // Number of endpoints in this intf
        0x00,			// Class code
        0x00,			// Subclass code
        0x00,			// Protocol code
        5                       // Interface string index
    },
    /* Endpoint Descriptor */
    { sizeof(USB_EP_DSC), DSC_EP, _EP02_OUT, _BULK, USER_EP_SIZE, 0x00 },
    { sizeof(USB_EP_DSC), DSC_EP, _EP02_IN,  _BULK, USER_EP_SIZE, 0x00 },

};

/* String Descriptors */
const struct { byte l; byte t; word s[1]; } sd000 =
{
    sizeof(sd000),
    DSC_STR,
    { 0x0409 }
};

const struct { byte l; byte t; word s[21]; } sd001 =
{
    sizeof(sd001),
    DSC_STR,
    { 'I','l','l','e','g','a','l',' ','S','y','s','t','e','m','s',' ','C','o','r','p','.' }
};

const struct { byte l; byte t; word s[24]; } sd002 =
{
    sizeof(sd002),
    DSC_STR,
    { 'A','u','d','i',' ','G','a','m','m','a',' ','I','I','I',' ','i','n','t','e','r','f','a','c','e' }
};

const struct { byte l; byte t; word s[5]; } sd003 =
{
    sizeof(sd003),
    DSC_STR,
    { 'V','1','.','0','1' }
};

const struct { byte l; byte t; word s[8]; } sd004 =
{
    sizeof(sd004),
    DSC_STR,
    { 'K','e','y','b','o','a','r','d' }
};

const struct { byte l; byte t; word s[5]; } sd005 =
{
    sizeof(sd005),
    DSC_STR,
    { 'R','a','d','i','o' }
};

const struct _dlist string_dsc[] =
{
  { &sd000, sizeof(sd000) },
  { &sd001, sizeof(sd001) },
  { &sd002, sizeof(sd002) },
  { &sd003, sizeof(sd003) },
  { &sd004, sizeof(sd004) },
  { &sd005, sizeof(sd005) }
};

const byte hid_rpt00[] =
{
	0x05, 0x01,		// Usage Page (Generic Desktop),
	0x09, 0x06,		// Usage (Keyboard),
	0xA1, 0x01,		// Collection (Application),

	0x05, 0x07,		//     Usage Page (Key Codes);
	0x19, 0xE0,		//     Usage Minimum (224),
	0x29, 0xE7,		//     Usage Maximum (231),
	0x15, 0x00,		//     Logical Minimum (0),
	0x25, 0x01,		//     Logical Maximum (1),
	0x75, 0x01,		//     Report Size (1),
	0x95, 0x08,		//     Report Count (8),
	0x81, 0x02,		//     Input (Data, Variable, Absolute),   ; Modifier byte

	0x95, 0x01,		//     Report Count (1),
	0x75, 0x08,		//     Report Size (8),
	0x81, 0x01,		//     Input (Constant),                   ; Reserved byte

	0x95, 0x05,		//     Report Count (5),
	0x75, 0x01,		//     Report Size (1),
	0x05, 0x08,		//     Usage Page (Page# for LEDs),
	0x19, 0x01,		//     Usage Minimum (1),
	0x29, 0x05,		//     Usage Maxmimum (5),
	0x91, 0x02,		//     Output (Data, Variable, Absolute),  ; LED report

	0x95, 0x01,		//     Report Count (1),
	0x75, 0x03,		//     Report Size (3),
	0x91, 0x01,		//     Output (Constant),                  ; LED report padding

	0x95, 0x06,		//     Report Count (6),
	0x75, 0x08,		//     Report Size (8),
	0x15, 0x00,		//     Logical Minimum (0),
	0x26, 0xff, 0x00,	//     Logical Maximum (256),
	0x05, 0x07,		//     Usage Page (Key Codes),
	0x19, 0x00,		//     Usage Minimum (0),
	0x29, 0xff,		//     Usage Maximum (256),
	0x81, 0x00,		//     Input (Data, Array),                ; Key arrays (6 bytes)

	0xC0			// End Collection
};

const struct _dlist hid_rpt[] =
{
    { &hid_rpt00, sizeof(hid_rpt00) }
};

const struct _dlist hid_dsc[] =
{
    { &cfg01.hid_i00a00, sizeof(USB_HID_DSC) }
};
