/***************************************************************************
 *   Copyright (C) 2009 by Stanson <stanson@pangolin.ru>                   *
 *   GPL v3 code, read COPYING file                                        *
 **************************************************************************/
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <limits.h>
#include <signal.h>
#include <ctype.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/ioctl.h>

#include <getopt.h>

#include <usb.h>

#define SEG_A	0x40
#define SEG_B	0x20
#define SEG_C	0x10
#define SEG_D	0x08
#define SEG_E	0x04
#define SEG_F	0x01
#define SEG_G	0x02
#define SEG_DP	0x80

unsigned char seg7[] = {
/* 0 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,
/* 1 */		SEG_B | SEG_C,
/* 2 */		SEG_A | SEG_B | SEG_G | SEG_E | SEG_D,
/* 3 */		SEG_A | SEG_B | SEG_G | SEG_C | SEG_D,
/* 4 */		SEG_B | SEG_C | SEG_F | SEG_G,
/* 5 */		SEG_A | SEG_F | SEG_G | SEG_C | SEG_D,
/* 6 */		SEG_A | SEG_F | SEG_G | SEG_C | SEG_D | SEG_E,
/* 7 */		SEG_A | SEG_B | SEG_C,
/* 8 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,
/* 9 */		SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G
};

int main(int argc, char *argv[])
{int			ret;
 char			*prog;

 static struct option long_options[] =
 {
   {"version", no_argument,       0, 'v'},
   {"device",  required_argument, 0, 'd'},
   {"write",   required_argument, 0, 'w'},
   {"radio",   required_argument, 0, 'r'},
   {"am",      no_argument,       0, 'a'},
   {"echo",    required_argument, 0, 'e'},
   {"listen",  no_argument,       0, 'l'},
   {"help",    no_argument,       0, 'h'},
   {0, 0, 0, 0}
 };

    int			iface = 0;
    char		*product, *vendor, *id = "dead:0034";
    unsigned short	vendor_id  = 0xdead;
    unsigned short	product_id = 0x0034;
    struct usb_bus 	*usbbus;
    struct usb_device	*usbdev;
    usb_dev_handle	*dev;

    char		data[64];
    unsigned long long 	ull = 0;
    unsigned long bit = 0;
    char		echo[64] = { 0 };
    unsigned long long	freq = 0;
    int am = 0;
    int listen = 0;

	prog = strrchr(argv[0],'/');
	if(!prog) prog = argv[0];
	else      prog++;

	while ((ret = getopt_long (argc, argv, "d:vaw:r:e:lh", long_options, NULL)) != -1)
	{
		switch(ret)
		{
    			case 'd':
			    id = strdup(optarg);
			    product = optarg;
    			    vendor = strsep(&product, ":");
			    if(vendor && *vendor) {
				vendor_id = strtol(vendor, NULL, 16);
			    }
			    if(product && *product) {
				product_id = strtol(product, NULL, 16);
			    }
			    break;
			case 'w': bit = strtoul( optarg, NULL, 10 ); break;
			case 'e': strncpy( echo, optarg, sizeof(echo) ); break;
			case 'r': freq = strtoul( optarg, NULL, 10 ); break;
			case 'a': am = 1; break;
			case 'l': listen = 1; break;
			case 'v': break;
			case 'h':
			default:
				printf("Usage: %s [--list/-l] [--read/-r] [--erase/-e] [--write filename/-w filename] [--help/-h] [--version/-v]\n", prog);
				printf("Options:\n");
				printf("\t-d, --device vendor_id:product_id  Use device with vendor:product id\n");
				printf("\t-l, --list                         List known processors\n");
				printf("\t-r, --read                         Read processor memory\n");
				printf("\t-e, --erase                        Erase processor memory\n");
				printf("\t-w, --write filename               Write Intel HEX file into memory\n");
				printf("\t-v, --version                      Print bootloader version\n");
				printf("\t-h, --help                         Show this help\n");
				return 1;
		}
	}

    usb_init();
    usb_find_busses();
    usb_find_devices();

    usbbus = usb_get_busses();
    for (; usbbus; usbbus = usbbus->next)
    {
	for (usbdev = usbbus->devices; usbdev; usbdev=usbdev->next)
	{
	    if (usbdev->descriptor.idVendor == vendor_id &&
	        usbdev->descriptor.idProduct == product_id) 
	    {
		dev = usb_open(usbdev);
		if(!dev)
		{
		    fprintf(stderr, "Failed to open USB device %s\n", id);
		    ret = -1;
		    goto out;
		}
		ret = usb_claim_interface(dev, iface);
		if(ret < 0)
		{
		    ret = usb_detach_kernel_driver_np(dev, iface);
		    if(ret < 0)
		    {
			fprintf(stderr, "Failed to detach kernel driver from interface %d on device %s\n", iface, id);
			goto out_close;
		    }
		    ret = usb_claim_interface(dev, iface);
		    if(ret < 0)
		    {
			fprintf(stderr, "Failed to claim interface %d on device %s\n", iface, id);
			goto out_close;
		    }
		}
	        goto device_found;
	    }
        }
    }

    fprintf(stderr, "Device %s not found!\n", id);
    goto out;

device_found:

    usb_get_string_simple(dev, 2, data, sizeof(data));

    printf("Found %s\n", data);

    /* DX ST DL VF UP B1 B2 DN [U|M|V|I|L|_]n nnn.n */
    if( echo[0] )
    {
	char *t, *p;
	int i;
	unsigned long long d = 0;

	ull = 0;

	t = strtok_r( echo, " ", &p );
	while( t )
	{
	         if( !strncmp( t, "DX", 2 ) )	ull |= 1ULL << 8;
	    else if( !strncmp( t, "DL", 2 ) )	ull |= 1ULL << 23;
	    else if( !strncmp( t, "ST", 2 ) )	ull |= 1ULL << 9;
	    else if( !strncmp( t, "VF", 2 ) )	ull |= 1ULL << 60;
	    else if( !strncmp( t, "UP", 2 ) )	ull |= 1ULL << 62;
	    else if( !strncmp( t, "DN", 2 ) )	ull |= 1ULL << 63;
	    else if( !strncmp( t, "B1", 2 ) )	ull |= 1ULL << 15;
	    else if( !strncmp( t, "B2", 2 ) )	ull |= 1ULL << 14;
	    else if( strchr( "UVMIL_", t[0] ) && isdigit( t[1] ) )
	    {
		switch( t[0] )
		{
		    case 'U': ull |= 0x38ULL << 8; break;
		    case 'V': ull |= 0x04ULL << 8; break;
		    case 'M': ull |= 0x2CULL << 8; break;
		    case 'I': ull |= 0x20ULL << 8; break;
		    case 'L': ull |= 0x18ULL << 8; break;
		    case '_': ull |= 0x10ULL << 8; break;
		}
		ull |= (unsigned long long)seg7[ t[1] - '0' ] << 16;
	    }
	    else if( strchr("CDEFIR-=_", t[0] ) && t[1] == 0 )
	    {
		switch( t[0] )
		{
		    case 'C': ull |= 0x05ULL << 56; break;
		    case 'D': ull |= 0x0DULL << 56; break;
		    case 'E': ull |= 0x07ULL << 56; break;
		    case 'F': ull |= 0x03ULL << 56; break;
		    case 'I': ull |= 0x08ULL << 56; break;
		    case 'R': ull |= 0x0BULL << 56; break;
		    case '-': ull |= 0x02ULL << 56; break;
		    case '=': ull |= 0x06ULL << 56; break;
		    case '_': ull |= 0x04ULL << 56; break;
		}
	    }
	    else if( isdigit(t[0]) )
	    {
		for( i = 0; t[i]; i++ )
		{
		    if( t[i] == '.' )
		    {
			ull |= 1ULL << 47;
		    }
		    else if( isdigit( t[i] ) )
		    {
			d |= (unsigned long long)seg7[ t[i] - '0' ] << 56;
			d >>= 8;
		    }
		}
		ull |= d;
	    }

	    t = strtok_r( NULL, " ", &p );
	}

    }
    else if( freq )
    {
	ull = am ? 0x0000000000020101ULL : 0x0000000000030101ULL;
	ull |= ( freq & 0x7FFF ) << 24;
	printf("0x%016llX\n", ull);
    }
    else if( listen )
    {
	while(1)
	{
	    ret = usb_bulk_read( dev, USB_ENDPOINT_IN | 2, data, 2, 1000 );
	    if( ret == -ETIMEDOUT ) continue;
	    if( ret < 0 ) { printf( "usb_bulk_read error %d\n", ret ); break; }
	    printf( "Key %d %s\n", data[1], data[0] ? "Released" : "Pressed" );
	}
    }
    else
    {
	ull = 1;
	ull <<= bit;
    }

    usb_bulk_write(dev, USB_ENDPOINT_OUT | 2, (char *)&ull, 8, 1000);

out_close:
    usb_close(dev);
out:
    return ret;
}
